import datetime

__author__ = "PG"
__version__ = "1.0.1"
__description__ = "The Modern en/de-cryption CLI utility"
__copyright__ = f"Copyright {datetime.date.today().year} PG"
__github__ = "https://github.com/Programming-geek582/encipher"
__email__ = "htmlgeek752@gmail.com"



from .ui import initialise_cipher
from .utilities import get_input, encrypt_and_display_info, decrypt_and_display_info
from cryptography.fernet import Fernet

key = Fernet.generate_key()
fernet = Fernet(key)
key = key.decode('utf-8')

def main():
    initialise_cipher()
    encryptOrDecrypt = get_input('Do you wanna encrypt or decrypt text(encrypt, decrypt): ', ['encrypt', 'decrypt'])
    if encryptOrDecrypt.lower() == "encrypt":
        text = get_input('Enter the text to encrypt: ').encode()
        encrypt_and_display_info(fernet, key, text)
        
    elif encryptOrDecrypt.lower() == "decrypt":
        text = get_input('Enter the text to decrypt: ').encode()
        decrypt_and_display_info(text)



from click import secho
from . import __author__, __copyright__, __description__

def initialise_cipher():
	secho(f"""
  ____ _       _
 / ___(_)_ __ | |__   ___ _ __
| |   | | '_ \| '_ \ / _ \ '__|		{__description__}
| |___| | |_) | | | |  __/ |		
 \____|_| .__/|_| |_|\___|_|		{__copyright__}
        |_|
		""",
		fg="bright_red"
	)
	print(f"""\nThe private key is saved in the file called key 
so pls make sure to keep this file safe and also
Not to make any change to the file content or name.\n""")


import os
import time
from typing import Optional, List
from cryptography.exceptions import InvalidSignature, InvalidKey
from cryptography.fernet import Fernet

def get_input(prompt, options : Optional[List] = None):
    while 1:
        text = input(prompt)
        if options and text.lower() not in options:
            print('Invalid input, retry with a valid input')
            continue
    
        return text

def write_key(key):
    with open('fernetKey', 'w+') as f:
        f.truncate()
        f.write(key)

def load_key():
    if not os.path.isfile('fernetKey'):
        print('A key was not found, exiting the program...')
        time.sleep(2)
        exit()

    with open('fernetKey', 'r') as f:
        content = f.readlines()

    return content[0].encode()

def encrypt_and_display_info(fernet_object : Fernet, key, text : bytes):
    encrypted = fernet_object.encrypt(text)
    print(f'The encrypted text is: \n{encrypted.decode("utf-8")}')
    write_key(key)
    print('The private key was saved, dont change that of any sort...')

def decrypt_and_display_info(ciphertext : bytes):
    key = load_key()
    decryptor = Fernet(key)
    try:
        decrypted = decryptor.decrypt(ciphertext)
        print(f'Decrypted version is: {decrypted.decode("utf-8")}')
    except Exception as e:
        if isinstance(e, InvalidKey):
            print('Invalid key provided')
            return

        elif isinstance(e, InvalidSignature):
            print('The private key doesnt match the digitally signed key of the ciphertext provided.')
            return
    
        else:
            raise e


from setuptools import setup

import cipher

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="cipher-symmetric",
    author=cipher.__author__,
    author_email=cipher.__email__,
    version=cipher.__version__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=cipher.__github__,
    packages=["cipher"],
    entry_points={"console_scripts": ["cipher=cipher.__main__:main"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.8",
    ],
    python_requires=">=3.8",
    include_package_data=True,
    exclude=("__pycache__",)
)

